"""
Sample solution for the CS444 image classification task.  

This solution first finds the mean RGB values for every image in the
the training set.  It then classifies the images in the test set
according to the image in the training set with the closest average color.

This is not a very good solution!  It does have the advantages of
being fast, and performing better than random guessing.

author: Nathan Sprague
version: 10/2013
"""

import sklearn.neighbors 
import numpy as np
import pa3util

def convert_to_mean_rgb(data):
    """
    Compute the mean RGB values for a collection of images. 

    Parameters: 
        data - an n x 3072 numpy array, where each row represents an image

    Returns: 
        An n x 3 numpy array where each row contains the average color value 
        of the original image. 
    """
    rgb_data = np.empty((data.shape[0], 3))
    for row in range(data.shape[0]):
        img = pa3util.data_to_img(data[row,:])
        rgb_data[row,0] = np.mean(img[:,:,0])
        rgb_data[row,1] = np.mean(img[:,:,1])
        rgb_data[row,2] = np.mean(img[:,:,2])
    return rgb_data


def knn_classify(show_results=False):
    """
    Classify the test data using the 1-nearest neighbor algorithm.
    This method saves a .csv file in the appropriate format. 

    Parameter: 
    show_results - set this to True if you want to see which images
                   are being classified as people. 
    """
    
    # First load the data...
    train_data = np.load('data/train_data.npy')
    train_labels = np.load('data/train_labels.npy')
    test_data = np.load('data/test_data.npy')

    # "Train" the classifier... 
    knn = sklearn.neighbors.KNeighborsClassifier(1)
    knn.fit(convert_to_mean_rgb(train_data), train_labels)

    # Classify the test data and save the results. 
    predict = knn.predict(convert_to_mean_rgb(test_data))
    pa3util.generate_submission(predict, 'test_submission.csv')
    
    if show_results:
        pa3util.show_results(test_data, predict)
    
if __name__ == "__main__":
    knn_classify(True)
